from example_interfaces.action._fibonacci import Fibonacci  # noqa: F401
from example_interfaces.action._fibonacci import Fibonacci_GetResult_Event  # noqa: F401
from example_interfaces.action._fibonacci import Fibonacci_GetResult_Request  # noqa: F401
from example_interfaces.action._fibonacci import Fibonacci_GetResult_Response  # noqa: F401
from example_interfaces.action._fibonacci import Fibonacci_SendGoal_Event  # noqa: F401
from example_interfaces.action._fibonacci import Fibonacci_SendGoal_Request  # noqa: F401
from example_interfaces.action._fibonacci import Fibonacci_SendGoal_Response  # noqa: F401



from example_interfaces.srv._add_two_ints import AddTwoInts  # noqa: F401
from example_interfaces.srv._add_two_ints import AddTwoInts_Event  # noqa: F401
from example_interfaces.srv._add_two_ints import AddTwoInts_Request  # noqa: F401
from example_interfaces.srv._add_two_ints import AddTwoInts_Response  # noqa: F401
from example_interfaces.srv._set_bool import SetBool  # noqa: F401
from example_interfaces.srv._set_bool import SetBool_Event  # noqa: F401
from example_interfaces.srv._set_bool import SetBool_Request  # noqa: F401
from example_interfaces.srv._set_bool import SetBool_Response  # noqa: F401
from example_interfaces.srv._trigger import Trigger  # noqa: F401
from example_interfaces.srv._trigger import Trigger_Event  # noqa: F401
from example_interfaces.srv._trigger import Trigger_Request  # noqa: F401
from example_interfaces.srv._trigger import Trigger_Response  # noqa: F401



# generated from rosidl_generator_py/resource/_idl.py.em
# with input from example_interfaces:msg/UInt32MultiArray.idl
# generated code does not contain a copyright notice

# This is being done at the module level and not on the instance level to avoid looking
# for the same variable multiple times on each instance. This variable is not supposed to
# change during runtime so it makes sense to only look for it once.
from os import getenv

ros_python_check_fields = getenv('ROS_PYTHON_CHECK_FIELDS', default='')


# Import statements for member types

# Member 'data'
import array  # noqa: E402, I100

import builtins  # noqa: E402, I100

import rosidl_parser.definition  # noqa: E402, I100


class Metaclass_UInt32MultiArray(type):
    """Metaclass of message 'UInt32MultiArray'."""

    _CREATE_ROS_MESSAGE = None
    _CONVERT_FROM_PY = None
    _CONVERT_TO_PY = None
    _DESTROY_ROS_MESSAGE = None
    _TYPE_SUPPORT = None

    __constants = {
    }

    @classmethod
    def __import_type_support__(cls):
        try:
            from rosidl_generator_py import import_type_support
            module = import_type_support('example_interfaces')
        except ImportError:
            import logging
            import traceback
            logger = logging.getLogger(
                'example_interfaces.msg.UInt32MultiArray')
            logger.debug(
                'Failed to import needed modules for type support:\n' +
                traceback.format_exc())
        else:
            cls._CREATE_ROS_MESSAGE = module.create_ros_message_msg__msg__u_int32_multi_array
            cls._CONVERT_FROM_PY = module.convert_from_py_msg__msg__u_int32_multi_array
            cls._CONVERT_TO_PY = module.convert_to_py_msg__msg__u_int32_multi_array
            cls._TYPE_SUPPORT = module.type_support_msg__msg__u_int32_multi_array
            cls._DESTROY_ROS_MESSAGE = module.destroy_ros_message_msg__msg__u_int32_multi_array

            from example_interfaces.msg import MultiArrayLayout
            if MultiArrayLayout.__class__._TYPE_SUPPORT is None:
                MultiArrayLayout.__class__.__import_type_support__()

    @classmethod
    def __prepare__(cls, name, bases, **kwargs):
        # list constant names here so that they appear in the help text of
        # the message class under "Data and other attributes defined here:"
        # as well as populate each message instance
        return {
        }


class UInt32MultiArray(metaclass=Metaclass_UInt32MultiArray):
    """Message class 'UInt32MultiArray'."""

    __slots__ = [
        '_layout',
        '_data',
        '_check_fields',
    ]

    _fields_and_field_types = {
        'layout': 'example_interfaces/MultiArrayLayout',
        'data': 'sequence<uint32>',
    }

    # This attribute is used to store an rosidl_parser.definition variable
    # related to the data type of each of the components the message.
    SLOT_TYPES = (
        rosidl_parser.definition.NamespacedType(['example_interfaces', 'msg'], 'MultiArrayLayout'),  # noqa: E501
        rosidl_parser.definition.UnboundedSequence(rosidl_parser.definition.BasicType('uint32')),  # noqa: E501
    )

    def __init__(self, **kwargs):
        if 'check_fields' in kwargs:
            self._check_fields = kwargs['check_fields']
        else:
            self._check_fields = ros_python_check_fields == '1'
        if self._check_fields:
            assert all('_' + key in self.__slots__ for key in kwargs.keys()), \
                'Invalid arguments passed to constructor: %s' % \
                ', '.join(sorted(k for k in kwargs.keys() if '_' + k not in self.__slots__))
        from example_interfaces.msg import MultiArrayLayout
        self.layout = kwargs.get('layout', MultiArrayLayout())
        self.data = array.array('I', kwargs.get('data', []))

    def __repr__(self):
        typename = self.__class__.__module__.split('.')
        typename.pop()
        typename.append(self.__class__.__name__)
        args = []
        for s, t in zip(self.get_fields_and_field_types().keys(), self.SLOT_TYPES):
            field = getattr(self, s)
            fieldstr = repr(field)
            # We use Python array type for fields that can be directly stored
            # in them, and "normal" sequences for everything else.  If it is
            # a type that we store in an array, strip off the 'array' portion.
            if (
                isinstance(t, rosidl_parser.definition.AbstractSequence) and
                isinstance(t.value_type, rosidl_parser.definition.BasicType) and
                t.value_type.typename in ['float', 'double', 'int8', 'uint8', 'int16', 'uint16', 'int32', 'uint32', 'int64', 'uint64']
            ):
                if len(field) == 0:
                    fieldstr = '[]'
                else:
                    if self._check_fields:
                        assert fieldstr.startswith('array(')
                    prefix = "array('X', "
                    suffix = ')'
                    fieldstr = fieldstr[len(prefix):-len(suffix)]
            args.append(s + '=' + fieldstr)
        return '%s(%s)' % ('.'.join(typename), ', '.join(args))

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if self.layout != other.layout:
            return False
        if self.data != other.data:
            return False
        return True

    @classmethod
    def get_fields_and_field_types(cls):
        from copy import copy
        return copy(cls._fields_and_field_types)

    @builtins.property
    def layout(self):
        """Message field 'layout'."""
        return self._layout

    @layout.setter
    def layout(self, value):
        if self._check_fields:
            from example_interfaces.msg import MultiArrayLayout
            assert \
                isinstance(value, MultiArrayLayout), \
                "The 'layout' field must be a sub message of type 'MultiArrayLayout'"
        self._layout = value

    @builtins.property
    def data(self):
        """Message field 'data'."""
        return self._data

    @data.setter
    def data(self, value):
        if self._check_fields:
            if isinstance(value, array.array):
                assert value.typecode == 'I', \
                    "The 'data' array.array() must have the type code of 'I'"
                self._data = value
                return
            from collections.abc import Sequence
            from collections.abc import Set
            from collections import UserList
            from collections import UserString
            assert \
                ((isinstance(value, Sequence) or
                  isinstance(value, Set) or
                  isinstance(value, UserList)) and
                 not isinstance(value, str) and
                 not isinstance(value, UserString) and
                 all(isinstance(v, int) for v in value) and
                 all(val >= 0 and val < 4294967296 for val in value)), \
                "The 'data' field must be a set or sequence and each value of type 'int' and each unsigned integer in [0, 4294967295]"
        self._data = array.array('I', value)
